import MicroApp from "./micro/micro-app";
import MicroServer from "./micro/micro-server";
import MicroConfig from "./micro/micro-config";
export { MicroError } from "./micro/micro-error";
export { Req, ReqConfig } from "./micro/micro-http";

/**
 * 开发集成
 * http-server
 * grpc-server
 */
export abstract class Server extends MicroServer<MicroApp> {
    /**
     * response json
     * @param data 数据
     * @param code code
     * @param msg msc
     */
    protected json_res(data: any, code: number = 0, msg: string = "ok") {
        return {
            code: code,
            msg: msg,
            data: data
        };
    }
}

/**
 * app
 * 1.内部注册服务
 * 2.外部注入注册配置
 */
export abstract class App extends MicroApp {
    // 存储 server
    private microServers: { [serverName: string]: Server } = {};

    /**
     * 动态实例化 server
     * e.g : 服务必须继承 MicroServer
     * use : getServer(new XXServer)
     * @param server 服务
     */
    protected getServer<A extends Server>(server: new (app: App) => A): A {
        let micro_server = this.microServers[server.name];
        if (micro_server === undefined) {
            micro_server = new server(this);
            this.microServers[server.name] = micro_server;
        }
        return micro_server as A;
    }
}

/**
 * 启动服务
 */
export class AppStart {
    /**
     * default : localhost
     * @param app 加载的 app
     */
    static start<T extends App>(
        app: new (config: MicroConfig) => T,
        config: MicroConfig
    ): T {
        return new app(config);
    }
}

export { MicroConfig };
