import { curl, RequestOptions } from "urllib";

/**
 * 网络请求 client
 */
export class Req {
    /**
     * 配置
     */
    private config: ReqConfig = {
        contentType: "application/json",
        dataType: "json"
    };
    /**
     * 初始化 baseUrl
     * @param baseUrl xxx.com
     */
    constructor(public readonly baseUrl: string) {}

    /**
     * 创建 url
     * @param url xxx.com/url 的 url
     */
    private createUrl(url: string) {
        return `${this.baseUrl}/${url}`;
    }

    /**
     * 配置
     */
    configReq(config: ReqConfig) {
        this.config = config;
        this.config.contentType = "application/json";
    }

    /**
     * 数据请求
     * @param url xxx.com/url 的 url
     * @param method 方法
     * @param header default : application/json
     * @param data  数据
     * @param is_content : 是否以文本 Body 提交, 默认 false, true 是不以 data 提交；
     */
    async req(
        url: string,
        method: "GET" | "POST" | "DELETE" | "PUT",
        data?: any,
        is_content: boolean = false
    ) {
        this.config.method = method;
        if (data != undefined) {
            this.config.data = data;
        }
        if (is_content) {
            this.config.content = JSON.stringify(data);
        }
        const path = this.createUrl(url);
        console.log(path);
        return await curl(path, this.config);
    }
}

export interface ReqConfig extends RequestOptions {}
